/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

#ifndef _SYNCH_H
#define _SYNCH_H
#ident	"@(#)head.usr:synch.h	1.5.2.1"
#ident	"$Header: $"
/*
 * This file defines user level synchronization objects,
 * functions, and macros
 */

#include <thread.h>
#include <lwpsynch.h>

/*
 * The following are used for static allocation
 */
#define DEFAULTMUTEX    {{0,0},{0,0},USYNC_THREAD, {0,0},0,0}
#define SHAREDMUTEX     {{0,0},{0,0},USYNC_PROCESS,{0,0},0,0}
#define DEFAULTCV 	{{0},{0,0},USYNC_THREAD, {0,0}}
#define SHAREDCV        {{0},{0,0},USYNC_PROCESS,{0,0}}
#define DEFAULTSEMA     {0,USYNC_THREAD, 0,{0},{0}}
#define SHAREDSEMA 	{0,USYNC_PROCESS,0,{0},{0}}

/*
 * Tests on lock states.
 */
#define SEMA_HELD(x)            ((x)->__s_count <= 0)
#define MUTEX_HELD(x)		(LOCK_HELD(&(x)->__m_lmutex))


/*
 * The following values are assigned to the type field of all 
 * synchronization primitives.  
 */
#define	USYNC_THREAD	0	/* known only to threads in a process */
#define	USYNC_PROCESS	1	/* shared by different processes      */
#define USYNC_DESTROYED	2	/* the object has been "destroyed" */

/*
 * Read-write lock
 */
#define	RW_READER	(0)
#define	RW_WRITER	(1)

/*
 * read-write locks.
 * rwlock_t is defined for both types, USYNC_THREAD or USYNC_PROCESS.
 * Those members only for USYNC_PROCESS are: rw_lwpcond, rw_wrwakeup,
 * and rw_rdwakecnt; those only for USYNC_THREAD are: rw_cvqhead,
 * and rw_cvqtail.
 */

typedef struct __rwlock	rwlock_t;

/*
 * spin locks - the spin_t structure is identical to mutex structure to
 * allow future interoperability.
 */

typedef mutex_t spin_t;

/*
 * synchronization barriers
 */
typedef volatile struct barrier barrier_t;
typedef volatile struct barrier_spin barrier_spin_t;

struct barrier {
	mutex_t		b_lock;		/* controls access to this barrier */
	int		b_type;		/* USYNC_PROCESS or USYNC_THREAD */
	unsigned int	b_count;	/* number of threads to synchronize  */
	unsigned int	b_waiting;	/* number of thread currently waiting */
	unsigned int	b_generation;	/* so we wake up for the right reason */
	cond_t		b_cond;		/* for waiting threads   */
};

struct barrier_spin {
	spin_t		bs_lock;	/* controls access to this barrier */
	long		bs_type;	/* USYNC_DESTROYED when destroyed    */
	unsigned int	bs_count;	/* number of threads to synchronize  */
	unsigned int	bs_waiting;	/* number of thread currently waiting*/
	unsigned int	bs_generation;	/* so we wake up for the right reason */
};

#ifdef __cplusplus
extern "C" {
#endif

int	mutex_init(mutex_t *, int, void *);
int	mutex_lock(mutex_t *);
int	mutex_unlock(mutex_t *);
int	mutex_trylock(mutex_t *);
int	mutex_destroy(mutex_t *);
int	cond_init(cond_t *, int, void *);
int	cond_signal(cond_t *);
int	cond_broadcast(cond_t *);
int	cond_wait(cond_t *,mutex_t *);
int	cond_timedwait(cond_t *,mutex_t *,timestruc_t *);
int	cond_destroy(cond_t *);
int	sema_init(sema_t *, int, int, void *);
int	sema_wait(sema_t *);
int	sema_trywait(sema_t *);
int	sema_post(sema_t *);
int	sema_destroy(sema_t *);
int	rwlock_init(rwlock_t *, int, void *);
int	rw_rdlock(rwlock_t *);
int	rw_wrlock(rwlock_t *);
int	rw_unlock(rwlock_t *);
int	rw_tryrdlock(rwlock_t *);
int	rw_trywrlock(rwlock_t *);
int	rwlock_destroy(rwlock_t *);
int	rmutex_init(rmutex_t *, int, void *);
int	rmutex_lock(rmutex_t *);
int	rmutex_unlock(rmutex_t *);
int	rmutex_trylock(rmutex_t *);
int	rmutex_destroy(rmutex_t *);
int	_spin_init(spin_t *, void *);
void	_spin_unlock(spin_t *);
void	_spin_lock(spin_t *);
int	_spin_trylock(spin_t *);
int	_spin_destroy(spin_t *);
int	barrier_init(barrier_t *, int, int, void *);
int	barrier_wait(barrier_t *);
int	barrier_destroy(barrier_t *);
int	_barrier_spin_init(barrier_spin_t *, int, void *);
int	_barrier_spin_wait(barrier_spin_t *);
int	_barrier_spin_destroy(barrier_spin_t *);

#ifdef __cplusplus
}
#endif

#endif /*_SYNCH_H*/
